@@ -7,38 +7,44 @@
  */
 
 pub mod email;
+pub mod html;
 pub mod rss;
 pub mod twitter;
 
-use serde::Deserialize;
-
 pub use self::email::Email;
+pub use self::html::Html;
 pub use self::rss::Rss;
 pub use self::twitter::Twitter;
 
 use crate::error::Result;
+use crate::read_filter::ReadFilter;
 use crate::sink::Message;
 
+// TODO: add message history via responce id -> message id hashmap
+// TODO: add pretty name/hashtag and link here instead of doing it manually
+#[derive(Debug)]
 pub struct Responce {
 	pub id: Option<String>,
 	pub msg: Message,
 }
 
-#[derive(Deserialize, Debug)]
-#[serde(tag = "type", rename_all = "snake_case", deny_unknown_fields)]
+// TODO: add google calendar source. Google OAuth2 is already implemented :)
+#[derive(Debug)]
 pub enum Source {
+	Email(Email),
+	Html(Html),
 	Rss(Rss),
 	Twitter(Twitter),
-	Email(Email),
 }
 
 impl Source {
 	// TODO: try using streams instead of polling manually?
-	pub async fn get(&mut self, last_read_id: Option<String>) -> Result<Vec<Responce>> {
+	pub async fn get(&mut self, read_filter: &ReadFilter) -> Result<Vec<Responce>> {
 		match self {
 			Self::Email(x) => x.get().await,
-			Self::Rss(x) => x.get(last_read_id).await,
-			Self::Twitter(x) => x.get(last_read_id).await,
+			Self::Html(x) => x.get(read_filter).await,
+			Self::Rss(x) => x.get(read_filter).await,
+			Self::Twitter(x) => x.get(read_filter).await,
 		}
 	}
 }
