@@ -10,9 +10,8 @@ use crate::auth::GoogleAuth;
 use crate::error::Result;
 
 pub enum Auth {
-	// TODO: use securestr or something of that sort
-	Password(String),
 	GoogleAuth(GoogleAuth),
+	Password(String),
 }
 
 pub(super) struct ImapOAuth2<'a> {
@@ -30,12 +29,12 @@ impl imap::Authenticator for ImapOAuth2<'_> {
 
 #[async_trait::async_trait]
 pub(super) trait GoogleAuthExt {
-	async fn to_imap_oauth2<'a>(&'a mut self, email: &'a str) -> Result<ImapOAuth2<'a>>;
+	async fn as_imap_oauth2<'a>(&'a mut self, email: &'a str) -> Result<ImapOAuth2<'a>>;
 }
 
 #[async_trait::async_trait]
 impl GoogleAuthExt for GoogleAuth {
-	async fn to_imap_oauth2<'a>(&'a mut self, email: &'a str) -> Result<ImapOAuth2<'a>> {
+	async fn as_imap_oauth2<'a>(&'a mut self, email: &'a str) -> Result<ImapOAuth2<'a>> {
 		Ok(ImapOAuth2 {
 			email,
 			token: self.access_token().await?,
