@@ -8,16 +8,14 @@
 
 use egg_mode::entities::MediaType;
 use egg_mode::{auth::bearer_token, tweet::user_timeline, KeyPair, Token};
-use serde::Deserialize;
 
-use crate::config;
-use crate::error::{Error, Result};
+use crate::error::Result;
+use crate::read_filter::ReadFilter;
+use crate::sink::message::{Link, LinkLocation};
 use crate::sink::Media;
 use crate::sink::Message;
 use crate::source::Responce;
 
-#[derive(Deserialize)]
-#[serde(try_from = "config::Twitter")]
 pub struct Twitter {
 	pretty_name: String, // used for hashtags
 	handle: String,
@@ -28,7 +26,7 @@ pub struct Twitter {
 }
 
 impl Twitter {
-	#[tracing::instrument(skip(api_key, api_secret))]
+	#[tracing::instrument(name = "Twitter::new", skip(api_key, api_secret))]
 	pub fn new(
 		pretty_name: String,
 		handle: String,
@@ -47,28 +45,28 @@ impl Twitter {
 		})
 	}
 
-	#[tracing::instrument]
-	pub async fn get(&mut self, last_read_id: Option<String>) -> Result<Vec<Responce>> {
+	#[tracing::instrument(name = "Twitter::get")]
+	pub async fn get(&mut self, read_filter: &ReadFilter) -> Result<Vec<Responce>> {
+		tracing::debug!("Getting tweets");
 		if self.token.is_none() {
 			self.token = Some(
-				bearer_token(&KeyPair::new(self.api_key.clone(), self.api_secret.clone()))
-					.await
-					.map_err(Error::TwitterAuth)?,
+				bearer_token(&KeyPair::new(self.api_key.clone(), self.api_secret.clone())).await?
+					// .await
+					// .map_err(Error::TwitterAuth)?,
 			);
 		}
 		// unwrap NOTE: initialized just above, should be safe
 		let token = self.token.as_ref().unwrap();
 
+		// TODO: keep a tweet id -> message id hashmap and handle enable with_replies from below
 		let (_, tweets) = user_timeline(self.handle.clone(), false, true, token) // TODO: remove clone
-			.older(last_read_id.as_ref().and_then(|x| x.parse().ok()))
+			.older(read_filter.last_read().and_then(|x| x.parse().ok()))
 			.await?;
 
-		// if !tweets.is_empty() {
-		// 	tracing::info!(
-		// 		"Got {amount} unread & unfiltered tweets",
-		// 		amount = tweets.len()
-		// 	);
-		// }
+		tracing::debug!(
+			"Got {num} tweets older than the last read one",
+			num = tweets.len()
+		);
 
 		let messages = tweets
 			.iter()
@@ -80,15 +78,27 @@ impl Twitter {
 					return None;
 				}
 
-				let text = format!(
-					"#{}\n\n{}\n<a href=\"https://twitter.com/{}/status/{}\">Link</a>",
-					self.pretty_name, tweet.text, self.handle, tweet.id
-				);
+				// let text = format!(
+				// 	"#{}\n\n{}\n<a href=\"\">Link</a>",
+				// 	self.pretty_name, tweet.text, self.handle, tweet.id
+				// );
 
 				Some(Responce {
 					id: Some(tweet.id.to_string()),
 					msg: Message {
-						text,
+						title: None,
+						body: tweet.text.clone(),
+						link: Some(Link {
+							url: format!(
+								"https://twitter.com/{handle}/status/{id}",
+								handle = self.handle,
+								id = tweet.id
+							)
+							.as_str()
+							.try_into()
+							.unwrap(),
+							loc: LinkLocation::Bottom,
+						}),
 						media: tweet.entities.media.as_ref().and_then(|x| {
 							x.iter()
 								.map(|x| match x.media_type {
@@ -107,6 +117,11 @@ impl Twitter {
 			})
 			.collect::<Vec<_>>();
 
+		tracing::debug!(
+			"{num} tweets remaining after filtering",
+			num = messages.len()
+		);
+
 		Ok(messages)
 	}
 
